#!/usr/local/bin/python3

import sys, re, json
import pathlib
import argparse
from ruamel.yaml import YAML
from os import path as path
from os import scandir as scandir
from os import remove as deleteFile
from collections import OrderedDict

here_path = path.dirname( path.abspath(__file__) )
this_script = re.sub(".py", ".yaml", path.basename( __file__ ) )
yaml = YAML()
yaml.indent(mapping=2, sequence=4, offset=2)
with open(path.join( here_path, "config", this_script), 'r') as c_f:
    config = yaml.load( c_f )

"""podmd
The script converts files in directories between yaml <-> json
It requires the specification of input and output directories and file formats.
podmd"""

################################################################################
################################################################################
################################################################################

def _get_args():

    parser = argparse.ArgumentParser()
    parser.add_argument("-i", "--inpath", help="File or directory to be converted.", required=True)
    parser.add_argument("-o", "--outpath", help="Directory to be saved to.", required=True)
    parser.add_argument("-t", "--filetype", help="File type to convert from; for directory parsing.", required=True)
    parser.add_argument("-x", "--exporttype", help="File type to convert to.", required=True)
    parser.add_argument("-v", "--verbose", help="Show conversion details.", default=False)

    return parser.parse_args()

################################################################################

def main():

    args = _get_args()
    in_path = pathlib.Path(args.inpath)
    out_path = pathlib.Path(args.outpath)
    from_type = args.filetype
    to_type = args.exporttype

    config.update({ "args": args, "from_type": from_type, "to_type": to_type} )

    for p in [in_path, out_path]:
        if not p.is_dir():
            print("¡¡¡ WARNING: No directory in {}!!!".format(p))
            exit()

    for t in [from_type, to_type]:
        if t not in config["supported_types"]:
            print("¡¡¡ WARNING: Only {} are supported!!!".format(join(config["supported_types"])))
            exit()

    convert_dir(in_path, out_path, from_type, to_type, config)

################################################################################

def convert_dir(in_path, out_path, from_type, to_type, config):

    pathlib.Path(out_path).mkdir( exist_ok=True )

    if config["delete_existing_files"] is True:
        for f in scandir(out_path):
            if to_type in pathlib.Path(f).suffix and f.is_file():
                deleteFile(f)

    fs = [ f.name for f in scandir(in_path) if f.is_file() ]
    fs = [ f for f in fs if f.endswith(from_type) ]

    conversion_method = "_{}2{}".format(from_type, to_type)

    for f_n in fs:
        eval(conversion_method)(f_n, in_path, out_path, config)

    d_s = [ d.name for d in scandir(in_path) if d.is_dir() ]
    for d in d_s:
        in_d = path.join( in_path, d )
        out_d = path.join( out_path, d )

        convert_dir(in_d, out_d, from_type, to_type, config)

################################################################################

def _yaml2json(f_n, in_path, out_path, config):

    in_file = path.join( in_path, f_n)
    o_n = re.sub(r"\.yaml", ".json", f_n)
    out_file = path.join( out_path, o_n)

    _file_conversion_message(config, in_file, out_file)
    i_d = _file_read_and_clean(in_file, config)

    try:
        s = yaml.load( i_d )
    except Exception as e:
        print("\n¡¡¡¡¡ ###########################\n{}".format(in_file))
        print(e)
        print("########################### !!!!!\n")
        return()

    _par_replace(s, config)
    with open(out_file, 'w') as out_f:
        out_f.write(json.dumps(OrderedDict(s), indent=4, sort_keys=True, default=str))

################################################################################

def _yaml2yaml(f_n, in_path, out_path, config):

    in_file = path.join( in_path, f_n)
    o_n = f_n
    out_file = path.join( out_path, f_n)

    _file_conversion_message(config, in_file, out_file)
    i_d = _file_read_and_clean(in_file, config)

    try:
        s = yaml.load( i_d )
    except Exception as e:
        _file_conversion_error(e, in_file)
        return

    _par_replace(s, config)
    with open(out_file, 'w') as out_f:
        yaml.dump(s, out_f)

################################################################################

def _json2yaml(f_n, in_path, out_path, config):

    in_file = path.join( in_path, f_n)
    o_n = re.sub(r"\.json", ".yaml", f_n)
    out_file = path.join( out_path, o_n)

    _file_conversion_message(config, in_file, out_file)
    i_d = _file_read_and_clean(in_file, config)

    try:
        s = json.loads(i_d)
    except Exception as e:
        _file_conversion_error(e, in_file)
        return

    _par_replace(s, config)
    with open(out_file, 'w') as out_f:
        yaml.dump(s, out_f)

################################################################################

def _json2json(f_n, in_path, out_path, config):

    in_file = path.join( in_path, f_n)
    o_n = f_n
    out_file = path.join( out_path, o_n)

    _file_conversion_message(config, in_file, out_file)
    i_d = _file_read_and_clean(in_file, config)
    try:
        s = json.loads(i_d)
    except Exception as e:
        _file_conversion_error(e, in_file)
        return

    _par_replace(s, config)
    with open(out_file, 'w') as out_f:
        yaml.dump(s, out_f)

################################################################################

def _file_read_and_clean(in_file, config):

    with open(in_file, 'r') as in_f:
        f_t = in_f.read()
        in_f.close()
    
    f_t = _text_replace(f_t, config)

    return f_t

################################################################################

def _text_replace(text, config):

    for r in config["replace_text"]:
        text = text.replace(r["from"], r["to"])

    return text

################################################################################

def _par_replace(schema, config):

    for r, rv in config["replace_vals"].items():
        if r in schema:
            schema.update({r: rv["replaceValue"]})
        else:
            if "add" in r:
                if rv["add"] is True:
                    schema.update({r: rv["replaceValue"]})

    return schema

################################################################################

def _file_conversion_error(e, f):

    print("\n¡¡¡¡¡ ######################################################\n{}".format(f))
    print(e)
    print("###################################################### !!!!!\n")

    return

################################################################################

def _file_conversion_message(config, in_file, out_file):

    if config["args"].verbose is False:
        return 

    print("converting {}\n        => {}".format(in_file, out_file))
    return

################################################################################
################################################################################
################################################################################

if __name__ == '__main__':
    main(  )



# __init__.py
from os import pardir, path
import sys, inspect

pkg_path = path.dirname( path.abspath(__file__) )
bycon_lib_path = path.join( pkg_path, "lib" )
sys.path.append( bycon_lib_path )

from aggregator_utils import *
from args_parsing import *
from cgi_parsing import *
from data_retrieval import *
from datatable_utils import *
from handover_generation import *
from interval_utils import *
from filter_parsing import *
from geomap_utils import *
from publication_utils import *
from query_execution import *
from query_generation import *
from read_specs import *
from response_remapping import *
from schema_parsing import *
from service_utils import *
from export_file_generation import *
from variant_parsing import *

c_f = Path( path.join( pkg_path, "config", "config.yaml" ) )
config = load_yaml_empty_fallback( c_f )
byc = initialize_bycon(config)
d_f = Path( path.join( pkg_path, "config", "beacon_defaults.yaml" ) )
byc.update({"beacon_defaults": load_yaml_empty_fallback( d_f ) })
for d_k, d_v in byc["beacon_defaults"]["defaults"].items():
    byc.update( { d_k: d_v } )

read_bycon_definition_files(byc)
cgi_parse_query(byc)



import re, yaml, json
from pymongo import MongoClient
from os import path, pardir, scandir
from pathlib import Path
from json_ref_dict import RefDict, materialize
from humps import camelize, decamelize

################################################################################

def read_bycon_definition_files(byc):

    if not "bycon_definition_files" in byc["config"]:

        config_d = path.join( byc["pkg_path"], "config" )

        b_d_fs = [ f.name for f in scandir(config_d) if f.is_file() ]
        b_d_fs = [ f for f in b_d_fs if f.endswith("yaml") ]
        b_d_fs = [ Path(f).stem for f in b_d_fs if not f.startswith("config.yaml") ]

        byc["config"].update({"bycon_definition_files": b_d_fs})

    for d in byc["config"]["bycon_definition_files"]:
        read_bycon_configs_by_name( d, byc )

    return byc

################################################################################
  
def read_bycon_configs_by_name(name, byc):

    """podmd
    Reading the config from the same wrapper dir:
    module
      |
      |- lib - read_specs.py
      |- config - __name__.yaml
    podmd"""

    o = {}
    ofp = path.join( byc["pkg_path"], "config", name+".yaml" )

    with open( ofp ) as od:
        o = yaml.load( od , Loader=yaml.FullLoader)

    byc.update({ name: o })

    return byc

################################################################################

def read_local_prefs(service, dir_path, byc):

    # We use snake_case in the paths
    service = decamelize(service)

    d = Path( path.join( dir_path, "config", service ) )

    # old style named config
    f = Path( path.join( dir_path, "config", service+".yaml" ) )

    if f.is_file():
        byc.update({"service_config": load_yaml_empty_fallback( f ) })
        return byc

    elif d.is_dir():
        t_f_p = Path( path.join( d, "config.yaml" ) )
        if t_f_p.is_file():
            byc.update({ "service_config": load_yaml_empty_fallback(t_f_p) } )

    return byc   

################################################################################

def dbstats_return_latest(byc):

    limit = 1
    if "stats_number" in byc:
        if byc["stats_number"] > 1:
            limit = byc["stats_number"]

    db = byc[ "config" ][ "info_db" ]
    coll = byc[ "config" ][ "beacon_info_coll" ]

    stats = MongoClient( )[ db ][ coll ].find( { }, { "_id": 0 } ).sort( "date", -1 ).limit( limit )
    return stats

################################################################################

def datasets_update_latest_stats(byc, collection_type="datasets"):

    results = [ ]

    def_k = re.sub(r's$', "_definitions", collection_type)
    q_k = re.sub(r's$', "_ids", collection_type)

    stat = dbstats_return_latest(byc)[0]

    for coll_id, coll in byc[ def_k ].items():
        if q_k in byc:
            if len(byc[ q_k ]) > 0:
                if not coll_id in byc[ q_k ]:
                    continue

        if collection_type in stat:
            if coll_id in stat[ collection_type ].keys():
                ds_vs = stat[ collection_type ][coll_id]
                if "counts" in ds_vs:
                    for c, c_d in byc["config"]["beacon_counts"].items():
                        if c_d["info_key"] in ds_vs["counts"]:
                            coll["info"].update({ c: ds_vs["counts"][ c_d["info_key"] ] })
                if "filtering_terms" in byc["response_entity_id"]:
                    coll.update({ "filtering_terms": stat[ collection_type ][coll_id]["filtering_terms"] } )

        results.append(coll)

    return results

################################################################################

def load_yaml_empty_fallback(yp):

    y = { }

    try:
        with open( yp ) as yd:
            y = yaml.load( yd , Loader=yaml.FullLoader)
    except:
        pass

    return y



import re, statistics
from progress.bar import Bar
import numpy as np

from cytoband_utils import *
from cgi_parsing import prjsoncam

################################################################################

"""
The methods here address genomic binning, the assignment of sample-specific
bin values (e.g. CNV overlap, maximum/minimum observed value in the bin...) as
well as the calculation of bin statistics.

The methods rely on the existence of cytoband files which contain information
about the individual chromosomes (size, centromer position as indicated by
the transition from p- to q-arm). The bins then dcan be generated directly
corresponding to the listed cytobands or (the default) by producing equally
sized bins (default 1Mb). The most distal bin of each arm then can be of a
different size.

Bin sizes are selected based on the provided key for a corresponding definition
in `interval_definitions.genome_bin_sizes` (e.g. 1Mb => 1000000).

### Interval Object Schema

```
no:
  description: counter, from 1pter -> Yqter (or whatever chromosomes are provided)
  type: integer
id:
  description: the id/label of the interval, from concatenating chromosome and base range
  type: string
reference_name:
  description: the chromosome as provided in the cytoband file
  type: string
  examples:
    - 7
    - Y
arm:
  type: string
  examples:
    - p
    - q
cytobands:
  type: string
  examples:
    - 1p12.1
start:
  description: the 0/interbase start of the interval
  type: integer
end:
  description: the 0/interbase end of the interval
  type: integer
```
"""

################################################################################
################################################################################
################################################################################

def generate_genomic_intervals(byc):

    if not "cytobands" in byc:
        parse_cytoband_file(byc)

    generate_cytoband_intervals(byc)

    binning = byc.get("genome_binning", "default")

    # cytobands ################################################################
    
    if binning == "cytobands":
        byc.update({"genomic_intervals": byc["cytoband_intervals"].copy() })
        return byc

    # otherwise intervals ######################################################

    c_l = byc["cytolimits"]
    i_d = byc["interval_definitions"]

    int_b = i_d["genome_bin_sizes"]["values"]["default"]
    bin_re = re.compile(r"^(\d+(\.\d+)?)(([kM])?b)?", re.IGNORECASE)

    assert binning in i_d["genome_bin_sizes"]["values"].keys(), '¡¡ Binning value "{}"" not in list !!'.format(binning)
    
    int_b = i_d["genome_bin_sizes"]["values"][binning]        
    
    e_p_f = i_d["terminal_intervals_soft_expansion_fraction"].get("value", 0.1)
    e_p = int_b * e_p_f

    intervals = []
    i = 1

    for chro in c_l.keys():

        p_max = c_l[chro]["p"][-1]
        q_max = c_l[chro]["size"]
        arm = "p"
        start = 0

        # calculate first interval to end p-arm with a full sized one
        p_first = p_max
        while p_first >= int_b + e_p:
            p_first -= int_b

        end = start + p_first
        while start < q_max:
            int_p = int_b
            if end > q_max:
                end = q_max
            elif q_max < end + e_p:
                end = q_max
                int_p += e_p
            if end >= p_max:
                arm = "q"
            size = end - start

            intervals.append( {
                    "no": i,
                    "id": "{}{}:{}-{}".format(chro, arm, start, end),
                    "reference_name": chro,
                    "arm": arm,
                    "cytobands": cytobands_label_from_positions(byc, chro, start, end),
                    "start": start,
                    "end": end,
                    "size": size
                })

            start = end
            end += int_p
            i += 1

    byc.update({ "genomic_intervals": intervals })

    return byc

################################################################################

def generate_cytoband_intervals(byc):

    intervals = []

    for cb in byc["cytobands"]:
        intervals.append( {
            "no": int(cb["i"]),
            "id": "{}:{}-{}".format(cb["chro"], cb["start"], cb["end"]),
            "reference_name": cb["chro"],
            "cytobands": cb["cytoband"],
            "start": int(cb["start"]),
            "end": int(cb["end"]),
            "size": int(cb["end"]) - int(cb[ "start"])
        })

    byc.update({ "cytoband_intervals": intervals })

    return byc

################################################################################

def interval_cnv_arrays(v_coll, query, byc):

    """
    The method generates sample-specific CNV maps using the currently defined
    interval bins. The output (`cnv_statusmaps`) provides annotated intervals
    for overlap fractions (`cnv_statusmaps.dup`, `cnv_statusmaps.del`) as well
    as the minimum and maximum values observed in those intervals
    (`cnv_statusmaps.max`, `cnv_statusmaps.min`).
    """

    v_d = byc["variant_definitions"]
    efo_vrs = v_d["efo_vrs_map"]
    c_l = byc["cytolimits"]
    intervals = byc["genomic_intervals"]

    cov_labs = { "DUP": 'dup', "DEL": 'del' }
    val_labs = { "DUP": 'max', "DEL": 'min' }

    int_no = len(intervals)

    maps = {
        "interval_count": int_no,
        "binning": byc["genome_binning"]
    }

    for cov_lab in cov_labs.values():
        maps.update({cov_lab: [0 for i in range(int_no)] })
    for val_lab in val_labs.values():
        maps.update({val_lab: [0 for i in range(int_no)] })

    cnv_stats = {
        "cnvcoverage": 0,
        "dupcoverage": 0,
        "delcoverage": 0,
        "cnvfraction": 0,
        "dupfraction": 0,
        "delfraction": 0       
    }

    chro_stats = {}

    for chro in c_l.keys():
        chro_stats.update({chro: cnv_stats.copy()})
        for arm in ['p', 'q']:
            c_a = chro+arm
            chro_stats.update({c_a: cnv_stats.copy()})

    v_s = v_coll.find( query )
    v_no = len(list(v_s))

    if v_no < 1:
        return maps, cnv_stats, chro_stats

    # the values_map collects all values for the given interval to retrieve
    # the min and max values of each interval
    values_map = [  [ ] for i in range(int_no) ]

    digests = []

    for v in v_s.rewind():

        if not "variant_state" in v:
            continue

        v_t_c = v["variant_state"].get("id", "__NA__")
        if v_t_c not in efo_vrs.keys():
            continue

        dup_del = efo_vrs[ v_t_c ]["DUPDEL"]
        if not dup_del in cov_labs.keys():
            continue
        cov_lab = cov_labs[dup_del]

        if not "reference_name" in v:
            v.update({"reference_name":v_d["refseq_chronames"][ v["location"]["sequence_id"] ]})

        v_i_id = v.get("variant_internal_id", None)
        v_cs_id = v.get("callset_id", None)

        if v_i_id in digests:
            print("¡¡¡ {} already counted for {} => deleting {}".format(v_i_id, v_cs_id, v["_id"]))
            v_coll.delete_one({"_id": v["_id"]})

        else:
            digests.append(v_i_id)

        for i, intv in enumerate(intervals):

            if _has_overlap(intv, v):

                ov_end = min(intv["end"], v["location"]["interval"]["end"]["value"])
                ov_start = max(intv["start"], v["location"]["interval"]["start"]["value"])
                ov = ov_end - ov_start
                maps[ cov_lab ][i] += ov

                try:
                    # print(type(v["info"]["cnv_value"]))
                    if type(v["info"]["cnv_value"]) == int or type(v["info"]["cnv_value"]) == float:
                        values_map[ i ].append(v["info"]["cnv_value"])
                    else:
                        values_map[ i ].append(v_d["cnv_dummy_values"][ dup_del ])
                except:
                    pass

    # statistics
    for cov_lab in cov_labs.values():
        for i, intv in enumerate(intervals):
            if maps[cov_lab][i] > 0:
                cov = maps[cov_lab][i]
                lab = cov_lab+"coverage"
                chro = str(intv["reference_name"])
                c_a = chro+intv["arm"]
                cnv_stats[ lab ] += cov
                chro_stats[chro][ lab ] += cov
                chro_stats[c_a][ lab ] += cov
                cnv_stats[ "cnvcoverage" ] += cov
                chro_stats[chro][ "cnvcoverage" ] += cov
                chro_stats[c_a][ "cnvcoverage" ] += cov
                maps[cov_lab][i] = round( maps[cov_lab][i] / intervals[ i ]["size"], 3 )

    for s_k in cnv_stats.keys():
        if "coverage" in s_k:
            f_k = re.sub("coverage", "fraction", s_k)
            cnv_stats.update({s_k: int(cnv_stats[ s_k ]) })
            cnv_stats.update({f_k: _round_frac(cnv_stats[ s_k ], byc["genome_size"], 3, f_k, v["callset_id"]) })

            for chro in c_l.keys():
                chro_stats[chro].update({s_k: int(chro_stats[chro][ s_k ]) })
                chro_stats[chro].update({f_k: _round_frac(chro_stats[chro][ s_k ], c_l[chro]['size'], 3, f_k+"_"+chro, v["callset_id"]) })
                for arm in ['p', 'q']:
                    c_a = chro+arm
                    s_a = c_l[chro][arm][-1] - c_l[chro][arm][0]
                    chro_stats[c_a].update({s_k: int(chro_stats[c_a][ s_k ]) })
                    chro_stats[c_a].update({f_k: _round_frac(chro_stats[c_a][ s_k ], s_a, 3, f_k+"_"+c_a, v["callset_id"]) })

    # the values for each interval are sorted, to allow extracting the min/max 
    # values by position
    # the last of the sorted values is assigned if > 0
    for i in range(len(values_map)):
        if values_map[ i ]:
            values_map[ i ].sort()
            if values_map[ i ][-1] > 0:
                maps["max"][i] = round(values_map[ i ][-1], 3)
            if values_map[ i ][0] < 0:
                maps["min"][i] = round(values_map[ i ][0], 3)

    return maps, cnv_stats, chro_stats

################################################################################

def _round_frac(val, maxval, digits=3, ftype="", label=""):

    f = round(val / maxval, digits)
    if f > 1:
        # print("!!! {} => {} fraction {}".format(label, ftype, f))
        f = 1
    return f

################################################################################

def interval_counts_from_callsets(callsets, byc):

    """
    This method will analyze a set (either list or MongoDB Cursor) of Progenetix
    callsets with CNV statusmaps and return a list of standard genomic interval
    objects with added per-interval quantitative data.
    """

    min_f = byc["interval_definitions"]["interval_min_fraction"].get("value", 0.001)
    int_fs = byc["genomic_intervals"].copy()
    int_no = len(int_fs)

    # callsets can be either a list or a MongoDB Cursor (which has to be re-set)
    if type(callsets).__name__ == "Cursor":
        callsets.rewind()

    cs_no = len(list(callsets))
    fFactor = 0

    if cs_no > 0:
        fFactor = 100 / cs_no;

    pars = {
        "gain": {"cov_l": "dup", "val_l": "max" },
        "loss": {"cov_l": "del", "val_l": "min" }
    }

    for t in pars.keys():

        covs = np.zeros( (cs_no, int_no) )
        vals = np.zeros( (cs_no, int_no) )

        if type(callsets).__name__ == "Cursor":
            callsets.rewind()

        for i, cs in enumerate(callsets):
            covs[i] = cs["cnv_statusmaps"][ pars[t]["cov_l"] ]
            vals[i] = cs["cnv_statusmaps"][ pars[t]["val_l"] ]

        counts = np.count_nonzero(covs >= min_f, axis=0)
        frequencies = np.around(counts * fFactor, 3)
        medians = np.around(np.ma.median(np.ma.masked_where(covs < min_f, vals), axis=0).filled(0), 3)
        means = np.around(np.ma.mean(np.ma.masked_where(covs < min_f, vals), axis=0).filled(0), 3)

        for i, interval in enumerate(int_fs):
            int_fs[i].update( {
                t+"_frequency": frequencies[i],
                t+"_median": medians[i],
                t+"_mean": means[i]
            } )

    return int_fs, cs_no

################################################################################

def _has_overlap(interval, v):

    if not interval["reference_name"] == v["reference_name"]:
        return False

    if not interval["end"] > v["location"]["interval"]["start"]["value"]:
        return False

    if not interval["start"] < v["location"]["interval"]["end"]["value"]:
        return False

    return True

################################################################################



#!/usr/local/bin/python3

import cgi, cgitb, sys, os, datetime, re, json, yaml
from os import environ, pardir, path
from pymongo import MongoClient

# local
pkg_path = path.join( path.dirname( path.abspath(__file__) ), pardir, pardir )
sys.path.append( pkg_path )
from bycon import *

"""podmd

* <https://progenetix.org/beacon/datasets/>

podmd"""

################################################################################
################################################################################
################################################################################

def main():

    cohorts()

################################################################################

def collections():

    cohorts()
    
################################################################################

def cohorts():

    initialize_service(byc)
    run_beacon_init_stack(byc)
    return_filtering_terms_response(byc)

    _return_cohorts_response(byc)

    run_result_sets_beacon(byc)
    query_results_save_handovers(byc)
    check_computed_interval_frequency_delivery(byc)
    check_switch_to_count_response(byc)
    check_switch_to_boolean_response(byc)
    cgi_print_response( byc, 200 )

################################################################################

def _return_cohorts_response(byc):

    if not "cohort" in byc["response_entity_id"]:
        return byc

    mongo_client = MongoClient( )

    cohorts =  []

    # TODO: verifier hack ...
    if not "cohorts" in byc["queries"]:
        byc.update({"test_mode": True})

    if byc["test_mode"] is True:
        byc["queries"].update( {"cohorts": { "collation_type": "pgxcohort" } } )

    try:
        query = byc["queries"]["cohorts"]
        
        for ds_id in byc[ "dataset_ids" ]:
            mongo_db = mongo_client[ ds_id ]        
            mongo_coll = mongo_db[ "collations" ]

            for cohort in mongo_coll.find( query ):
                cohorts.append(cohort)
                byc["service_response"]["response_summary"].update({"exists":True})
    except:
        pass

    cohorts = remap_cohorts(cohorts, byc)

    byc["service_response"]["response"].pop("result_sets", None)
    byc["service_response"]["response"].update({"collections": cohorts})
    byc["service_response"]["response_summary"].update({"num_total_results":len(cohorts)})
    cgi_print_response( byc, 200 )

################################################################################
################################################################################

if __name__ == '__main__':
    main()



#!/usr/local/bin/python3

import cgi, cgitb
import re, yaml
from os import path, pardir
import csv
import sys

pkg_path = path.join( path.dirname( path.abspath(__file__) ), pardir, pardir )
sys.path.append( pkg_path )
from bycon import *

################################################################################
################################################################################
################################################################################

def main():

    cytomapper()

################################################################################
################################################################################
################################################################################

def cytomapper():
    
    initialize_service(byc)
    
    parse_variant_parameters(byc)
    generate_genomic_intervals(byc)

    # response prototype
    create_empty_service_response(byc)

    cytoBands = [ ]
    if "cyto_bands" in byc["variant_pars"]:
        cytoBands, chro, start, end, error = bands_from_cytobands(byc["variant_pars"]["cyto_bands"], byc)
        byc["service_response"]["meta"]["received_request_summary"].update({ "cytoBands": byc["variant_pars"]["cyto_bands"] })
    elif "chro_bases" in byc["variant_pars"]:
        cytoBands, chro, start, end = bands_from_chrobases(byc["variant_pars"]["chro_bases"], byc)
        byc["service_response"]["meta"]["received_request_summary"].update({ "chroBases": byc["variant_pars"]["chro_bases"] })

    cb_label = cytobands_label( cytoBands )

    if len( cytoBands ) < 1:
        response_add_error(byc, 422, "No matching cytobands!" )
        cgi_break_on_errors(byc)

    size = int(  end - start )
    chroBases = "{}:{}-{}".format(chro, start, end)
    r_a = byc["variant_definitions"]["refseq_aliases"]
    sequence_id = r_a.get(chro, chro)

    if "text" in byc["output"]:
        open_text_streaming(byc["env"])
        print("{}\t{}".format(cb_label, chroBases))
        exit()

    # TODO: response objects from schema
    # r_s = byc["response_entity"]["beacon_schema"]["entity_type"]
    # cb_i = object_instance_from_schema_name(byc, r_s, "properties")
    
    results = [
        {
            "info": {
                "cytoBands": cb_label,
                "bandList": [x['chroband'] for x in cytoBands ],
                "chroBases": chroBases,
                "referenceName": chro,
                "size": size,
            },        
            "chromosome_location": {
                "type": "ChromosomeLocation",
                "species_id": "taxonomy:9606",
                "chr": chro,
                "interval": {
                    "start": cytoBands[0]["cytoband"],
                    "end": cytoBands[-1]["cytoband"],
                    "type": "CytobandInterval"
                }
            },
            "genomic_location": {
                "type": "SequenceLocation",
                "sequence_id": sequence_id,
                "interval": {
                    "start": {
                        "type": "Number",
                        "value": start
                    },
                    "end": {
                        "type": "Number",
                        "value": end
                    },
                    "type": "SequenceInterval"
                }
            }
        }
    ]

    populate_service_response( byc, results)
    cgi_print_response( byc, 200 )

################################################################################
################################################################################
################################################################################
################################################################################

if __name__ == '__main__':
    main()



import cgi, cgitb
import re, yaml
from pymongo import MongoClient

from cgi_parsing import *

################################################################################

def parse_filters(byc):

    if "filters" in byc["form_data"]:
        fs = byc["form_data"]["filters"]
        fs = check_filter_values(fs, byc)
        if len(fs) > 0:
            byc.update( { "filters": fs } )
            return byc
            
    return byc

################################################################################

def get_filter_flags(byc):

    ff = {
        "logic": byc[ "config" ][ "filter_flags" ][ "logic" ],
        "precision": byc[ "config" ][ "filter_flags" ][ "precision" ],
        "descendants": byc[ "config" ][ "filter_flags" ][ "include_descendant_terms" ]
    }

    if "form_data" in byc:
        if "filter_logic" in byc[ "form_data" ]:
            ff["logic"] = boolean_to_mongo_logic( byc["form_data"]['filter_logic'] )
        if "filter_precision" in byc[ "form_data" ]:
            ff["precision"] = byc["form_data"]['filter_precision']
        if "include_descendant_terms" in byc[ "form_data" ]:
            i_d_t = str(byc[ "form_data" ].get("include_descendant_terms", 1)).lower()
            if i_d_t in ["0", "-1", "no", "false"]:
                ff["descendants"] = False

    byc.update( { "filter_flags": ff } )

    return byc

################################################################################

def check_filter_values(filters, byc):

    f_defs = byc["filter_definitions"]

    checked = [ ]
    for f in filters:
        if not isinstance(f, dict):
            f = {"id":f}
        if not "id" in f:
            continue
        pre = re.split('-|:', f["id"])[0]
        for f_t, f_d in f_defs.items():
            if re.compile( f_d["pattern"] ).match( f["id"] ):       
                if f not in checked:
                    checked.append( f )

    return checked
  
################################################################################

def select_dataset_ids(byc):

    p_id = rest_path_value("datasets")
    if p_id:
        ds_id = p_id
        if ds_id in byc["dataset_definitions"].keys():
            byc.update( { "dataset_ids": [ ds_id ] } )
            return byc

    # accessid overrides ... ?
    if "accessid" in byc["form_data"]:
        accessid = byc["form_data"]["accessid"]

        ho_client = MongoClient()
        ho_db = ho_client[ byc["config"]["info_db"] ]
        ho_coll = ho_db[ byc["config"][ "handover_coll" ] ]
        h_o = ho_coll.find_one( { "id": accessid } )
        # TODO: catch error for mismatch
        if h_o:
            if "source_db" in h_o:
                ds_id = h_o["source_db"]
                if ds_id in byc["dataset_definitions"].keys():
                    byc.update( { "dataset_ids": [ ds_id ] } )
                    return byc            

    ds_ids = [ ]
    form = byc["form_data"]
    if "dataset_ids" in form:
        for ds_id in form["dataset_ids"]:
            if ds_id in byc["dataset_definitions"].keys():
                ds_ids.append(ds_id)
        if len(ds_ids) > 0:
            byc.update( { "dataset_ids": ds_ids } )
    
    return byc
  
################################################################################



"""A setuptools based setup module.
See:
https://packaging.python.org/guides/distributing-packages-using-setuptools/
https://github.com/pypa/sampleproject
"""

from setuptools import setup, find_packages
import pathlib

here = pathlib.Path(__file__).parent.resolve()
long_description = (here / "README.md").read_text(encoding="utf-8")

setup(
    name="bycon",
    version="1.0.10",
    description="A Python-based environment for the Beacon v2 genomics API",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/progenetix/bycon",
    author="Michael Baudis",
    author_email="contact@progenetix.org",
    classifiers=[
        "Intended Audience :: Developers",
        "Topic :: Internet :: WWW/HTTP :: Dynamic Content",
        "Topic :: Scientific/Engineering :: Bio-Informatics",
        "License :: CC0 1.0 Universal (CC0 1.0) Public Domain Dedication",
        "Topic :: Scientific/Engineering :: Interface Engine/Protocol Translator",
        "Topic :: Scientific/Engineering :: Medical Science Apps.",
        "Topic :: Software Development :: Libraries :: Python Modules",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3 :: Only"
    ],
    keywords="genomics, Beacon",
    package_dir={'':"."},
    packages=find_packages(where="."),  # Required
    setup_requires=['setuptools_scm'],
    include_package_data=True,
    package_data={'bycon': ['**/*.yaml']},
    python_requires=">=3.7, <4",
    # This field lists other packages that your project depends on to run.
    # Any package you put here will be installed by pip when your project is
    # installed, so they must be valid existing projects.
    #
    # For an analysis of "install_requires" vs pip's requirements files see:
    # https://packaging.python.org/discussions/install-requires-vs-requirements/
    install_requires=["base36", "isodate", "json_ref_dict", "liftover", "numpy", "progress", "pyhumps", "pymongo", "PyYAML", "ruamel.base"],  # Optional
    # Entry points. The following would provide a command called `sample` which
    # executes the function `main` from this package when invoked:
    # entry_points={  # Optional
    #     "console_scripts": [
    #         "sample=sample:main",
    #     ],
    # },
    # List additional URLs that are relevant to your project as a dict.
    #
    project_urls={  # Optional
        "Bug Reports": "https://github.com/progenetix/bycon/issues",
        "Source": "https://github.com/progenetix/bycon/",
    },
)


#!/usr/local/bin/python3

import cgi, cgitb
import re, json
from os import environ, path, pardir
import sys

# local
pkg_path = path.join( path.dirname( path.abspath(__file__) ), pardir, pardir )
sys.path.append( pkg_path )
from bycon import *

dir_path = path.dirname( path.abspath(__file__) )

"""podmd
The `ids` service forwards compatible, prefixed ids (see `config/ids.yaml`) to specific
website endpoints. There is no check if the id exists; this is left to the web
page handling itself.

Stacking with the "pgx:" prefix is allowed.
* <https://progenetix.org/services/ids/pgxbs-kftva5zv>
* <https://progenetix.org/services/ids/PMID:28966033>
* <https://progenetix.org/services/ids/NCIT:C3262>
podmd"""

################################################################################
################################################################################
################################################################################

def main():

    ids()
    
################################################################################

def ids():

    set_debug_state(debug=0)

    read_local_prefs( "ids", dir_path, byc )

    id_in = rest_path_value("ids")
    output = rest_path_value(id_in)

    if id_in:
        for f_p in byc["service_config"]["format_patterns"]:
            pat = re.compile( f_p["pattern"] )
            if pat.match(id_in):
                lid = id_in  
                link = f_p["link"]
                pim = f_p.get("prepend_if_missing", "")
                if len(pim) > 0:
                    if pim in lid:
                        pass
                    else:
                        lid = pim+lid

                print_uri_rewrite_response(link, lid, output)

    print('Content-Type: text')
    print('status:422')
    print()
    print("No correct id provided. Please refer to the documentation at http://info.progenetix.org/tags/services/")
    exit()

################################################################################
################################################################################

if __name__ == '__main__':
    main()
